; kbd.asm from rs_kbd project:
;  Keyboard initialization and main loop, low level kbd communications
;
; Copyright 2016,2017 Mihai Gaitos, mihaig@hawk.ro
; 
; Permission is hereby granted, without written agreement and without
; license or royalty fees, to use, copy, modify, and distribute this
; software and its documentation for any purpose, provided that the
; above copyright notice and the following two paragraphs appear in
; all copies of this software.
; 
; IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
; DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
; ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
; IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
; DAMAGE.
; 
; THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
; BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
; FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
; ON AN "AS IS" BASIS, AND THE COPYRIGHT HOLDER HAS NO OBLIGATION TO
; PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

INITKBD
		BSF		PORTB,K_DAT	; pull-up DATA
		BSF		PORTB,K_CLK ; and CLOCK
		; load initial values
		CLRF	KC
		CLRF	MYFLAGS
		CLRF	K_FLAGS

		BSF		STATUS,RP0
		BCF		TRISB,LRLED
		BCF		TRISB,UDLED
		BCF		STATUS,RP0

		BCF		PORTB,LRLED
		BSF		PORTB,UDLED


;		CALL	DELAY_BIG
;		CALL	DELAY_BIG
;		CALL	DELAY_BIG
;		MOVLW	H'FF'
;		CALL	SEND_KB
		BSF	PORTA,RDLED		; "READY" LED


MAINLOOP					; this also processes encoder and serial "events"
		BTFSC	PIR1,5		; PIR1 bit 5 is RCIF (receive complete flag)
		GOTO	RECRSBYTE	; if we got a BYTE on RS232 it cand be a (debug) command

		MOVF	PORTB,W
		CALL	CHKENC		; rotary ENCODER
		MOVF	ENCR,F		; if ENCR!=0
		BTFSC	STATUS,Z
		GOTO	OVER_ENC
		BTFSC	ENCR,0
		MOVLW	H'17'
		BTFSC	ENCR,1
		MOVLW	H'18'
		MOVWF	TMP_SC		; if we get an encoder position change,
		; simulate receiving a scan code, so that shift and ctrl will work
		; just as they would for regular keys
		CALL	SCPROC
OVER_ENC
		

		BTFSC	PORTB,K_CLK
		GOTO	MAINLOOP		; if RB3 is LOW, then KBD started generating CLOCK
		BCF		PORTA,RDLED		; BUSY :)

		 ; start working
		MOVLW	D'8'		; data bits..
		MOVWF	CNT_KB
		CLRF	TMP_SC
		BCF		MYFLAGS,KBWERR
KBDLOOP1
			CALL	KBWAITUPDN
			BTFSC	MYFLAGS,KBWERR
				GOTO	MAINLOOP
			RRF		PORTB,W		; get bit (KBD_DATA is bit 0)
			RRF		TMP_SC,1	; store it
			DECFSZ	CNT_KB,1
			GOTO	KBDLOOP1

		CALL	KBWAITUP	; next bit is parity
		BTFSC	MYFLAGS,KBWERR
			GOTO	MAINLOOP
		BCF		MYFLAGS,KBPAR	; clear parity, will be set if data is high
		CALL	KBWAITDN
		BTFSC	PORTB,K_DAT
			BSF		MYFLAGS,KBPAR	; store parity if 1
		BTFSC	MYFLAGS,KBWERR
			GOTO	MAINLOOP
		BTFSC	PORTB,K_DAT
		BSF		MYFLAGS,KBPAR
		CALL	KBWAITUP	; STOP BIT
		BTFSC	MYFLAGS,KBWERR
			GOTO	MAINLOOP
		CALL	KBWAITDN	; ignore, also
		BTFSC	MYFLAGS,KBWERR
			GOTO	MAINLOOP
		CALL	KBWAITUP	; wait until clock is high again (KB IDLE)

CHKPAR	; XOR all the 1 bits and the parity bit
		MOVF	TMP_SC,W
		MOVWF	TMPW
		MOVLW	D'8'
		MOVWF	CNT_KB
		MOVLW	1<<KBPAR 	; SET KBPAR bit in W, used to XOR the bit in MYFLAGS, below

CHK_PAR_LOOP
			RRF		TMPW,F
			BTFSC	STATUS,C
				XORWF	MYFLAGS,F
			DECFSZ	CNT_KB,F
			GOTO	CHK_PAR_LOOP
		BTFSC	MYFLAGS,KBPAR
		GOTO	PAR_OK	; if the result of the XORs is 1 then parity is OK
						; else parity error
		MOVLW	H'FE'	; request RESEND
		CALL	SEND_KB
		GOTO	MAINLOOP	; do not process erroneous scan code
PAR_OK
		CALL	KBINHIBIT
		include	"procsc.asm" ; process scancode

EOPSC
		CALL	KBRELEASE
		BSF		PORTA,RDLED		; "READY" LED
		GOTO	MAINLOOP

RECRSBYTE	; all commands are one byte (one char) long. call appropriate function based on char received
		MOVF	RCREG,W	; get BYTE
		BCF		PORTA,RDLED		; READY LED OFF
; inhibit KBD send data
		BCF	PORTB,K_CLK
		BSF	STATUS,RP0	;Switch to register bank 1
		BCF	TRISB,K_CLK
		BCF	STATUS,RP0	;back
		MOVWF	TMP_B
		CALL	SENDCHR
		MOVLW	"Z"
		SUBWF	TMP_B,W
		BTFSC	STATUS,Z
		GOTO	RESET
		MOVLW	"h"
		SUBWF	TMP_B,W
		BTFSC	STATUS,Z
		CALL	CHGMODE_TO_HEX
		MOVLW	"a"
		SUBWF	TMP_B,W
		BTFSC	STATUS,Z
		CALL	CHGMODE_TO_ASCII
		MOVLW	"t"
		SUBWF	TMP_B,W
		BTFSC	STATUS,Z
		CALL	MYTEST
		MOVLW	"x"
		SUBWF	TMP_B,W
		BTFSC	STATUS,Z
		CALL	MYTESTx
		MOVLW	"r"
		SUBWF	TMP_B,W
		BTFSC	STATUS,Z
		CALL	KRESET

		GOTO	EOPSC

STARTSCAN
		MOVLW	H'F4'
		CALL	SEND_KB
		RETURN

MYTEST
		MOVLW	H'EE'
		CALL	SEND_KB
		RETURN

MYTESTx
		MOVLW	H'00'
		MOVWF	NEXTKBB
		BSF		MYFLAGS,KBTOSEND
		MOVLW	H'F3'
		CALL	SEND_KB
		RETURN

KRESET
		MOVLW	H'FF'
		CALL	SEND_KB
		RETURN


KBWAITUP
		CLRF	CNTKBW
LKBWU
;		MOVLW	B'00001000'
;		XORWF	PORTA,F
		CALL	DRTRN
;		XORWF	PORTA,F
		INCFSZ	CNTKBW,F
		GOTO	BKBWU
		BSF		MYFLAGS,KBWERR
		RETURN
BKBWU
		BTFSS	PORTB,K_CLK
		GOTO	LKBWU
		RETURN


KBWAITDN
		CLRF	CNTKBW
LKBWD
;		MOVLW	B'00001000'
;		XORWF	PORTA,1
		CALL	DRTRN
;		XORWF	PORTA,1
		INCFSZ	CNTKBW,F
		GOTO	BKBWD
		BSF		MYFLAGS,KBWERR
		RETURN
BKBWD
		BTFSC	PORTB,K_CLK
		GOTO	LKBWD
		RETURN


KBINHIBIT
		BCF	PORTB,K_CLK
		BSF	STATUS,RP0		;Switch to register bank 1
		BCF	TRISB,K_CLK
		BCF	STATUS,RP0
		RETURN


KBRELEASE
		BSF	STATUS,RP0		;Switch to register bank 1
		BSF	TRISB,K_CLK		;RB3 is now INPUT
		BCF	STATUS,RP0	
		BSF	PORTB,K_CLK		; pull-up CLOCK
		RETURN

CALCPAR
		BSF		MYFLAGS,KBPAR
		MOVLW	D'8'
		MOVWF	CNT_KB
		MOVLW	1<<KBPAR ; used to XOR the bit in MYFLAGS, below
CALCPAR_LOOP
			RRF		TMP_SC,F
			BTFSC	STATUS,C
				XORWF	MYFLAGS,F
			DECFSZ	CNT_KB,F
			GOTO	CALCPAR_LOOP
		RRF		TMP_SC,F
		RETURN

SEND_KB
		MOVWF	TMP_SC
		MOVWF	BYTE4ACK
		BCF		MYFLAGS,KBWERR
		CALL	CALCPAR
		CALL	KBINHIBIT
		CALL	DELAY_100		; signal HOST-KBD comm by inhibiting CLOCK
		BCF		PORTB,K_DAT		; then SET DATA LOW (START BIT)
		BSF		STATUS,RP0
		BCF		TRISB,K_DAT
		BSF		TRISB,K_CLK		; and release clock
		BCF		STATUS,RP0
		BSF		PORTB,K_CLK
;		CALL	KBWAITUP
		MOVLW	D'8'			; send 8 data bits
		MOVWF	CNT_KB
SEND_KB_LOOP
			CALL	KBWAITUPDN		; WAIT a CLOCK CYCLE before sending each BIT
			BTFSC	MYFLAGS,KBWERR
				GOTO	SEND_ERR
			RRF		TMP_SC,F
			BTFSC	STATUS,C
				BSF		PORTB,K_DAT
			BTFSS	STATUS,C
				BCF		PORTB,K_DAT
			DECFSZ	CNT_KB,F
			GOTO	SEND_KB_LOOP

		CALL	KBWAITUPDN		; WAIT a CLOCK CYCLE
		BTFSC	MYFLAGS,KBWERR
			GOTO	SEND_ERR
		BTFSS	MYFLAGS,KBPAR	; then send parity bit
		BCF		PORTB,K_DAT
		BTFSC	MYFLAGS,KBPAR
		BSF		PORTB,K_DAT
		CALL	KBWAITUPDN		; after another CLOCK cycle
		BTFSC	MYFLAGS,KBWERR
			GOTO	SEND_ERR
		CALL	KBRELDAT		; release DATA


		CALL	KBWAITUPDN
		BTFSC	PORTB,K_DAT
			GOTO	SEND_NAK
		CALL	KBWAITUP
;		CALL	KBINHIBIT
;		CALL	DELAY_100
		
		RETURN




KBRELDAT
		BSF		STATUS,RP0		; release DATA
		BSF		TRISB,K_DAT
		BCF		STATUS,RP0
		BSF		PORTB,K_DAT
		RETURN


KBWAITUPDN		; only for SENDING!
		CALL	KBWAITUP
		BTFSC	MYFLAGS,KBWERR
			RETURN
		CALL	KBWAITDN
		RETURN
		
KBWAITDNUP		; only for SENDING!
		CALL	KBWAITDN
		BTFSC	MYFLAGS,KBWERR
			RETURN
		CALL	KBWAITUP
		RETURN
	
SEND_ERR
		; nothing, for now
		CALL	KBRELDAT	; just in case
		MOVLW	"!"
		CALL	SENDCHR
		RETURN

SEND_NAK
		CALL	KBINHIBIT
		MOVLW	"#"
		CALL	SENDCHR
		RETURN
